@@ -67,8 +67,8 @@ typedef struct {
 
 #ifdef _GLFW_X11
     int32_t                 keyboard_device_id;
-    GLFWbool                available;
-    GLFWbool                detectable;
+    bool                available;
+    bool                detectable;
     int                     majorOpcode;
     int                     eventBase;
     int                     errorBase;
@@ -79,18 +79,18 @@ typedef struct {
 } _GLFWXKBData;
 
 #ifdef _GLFW_X11
-GLFWbool glfw_xkb_set_x11_events_mask(void);
-GLFWbool glfw_xkb_update_x11_keyboard_id(_GLFWXKBData *xkb);
+bool glfw_xkb_set_x11_events_mask(void);
+bool glfw_xkb_update_x11_keyboard_id(_GLFWXKBData *xkb);
 #endif
 
 void glfw_xkb_release(_GLFWXKBData *xkb);
-GLFWbool glfw_xkb_create_context(_GLFWXKBData *xkb);
-GLFWbool glfw_xkb_compile_keymap(_GLFWXKBData *xkb, const char *map_str);
+bool glfw_xkb_create_context(_GLFWXKBData *xkb);
+bool glfw_xkb_compile_keymap(_GLFWXKBData *xkb, const char *map_str);
 void glfw_xkb_update_modifiers(_GLFWXKBData *xkb, xkb_mod_mask_t depressed, xkb_mod_mask_t latched, xkb_mod_mask_t locked, xkb_layout_index_t base_group, xkb_layout_index_t latched_group, xkb_layout_index_t locked_group);
-GLFWbool glfw_xkb_should_repeat(_GLFWXKBData *xkb, xkb_keycode_t scancode);
+bool glfw_xkb_should_repeat(_GLFWXKBData *xkb, xkb_keycode_t scancode);
 const char* glfw_xkb_keysym_name(xkb_keysym_t sym);
 xkb_keysym_t glfw_xkb_sym_for_key(int key);
 void glfw_xkb_handle_key_event(_GLFWwindow *window, _GLFWXKBData *xkb, xkb_keycode_t scancode, int action);
-int glfw_xkb_keysym_from_name(const char *name, GLFWbool case_sensitive);
+int glfw_xkb_keysym_from_name(const char *name, bool case_sensitive);
 void glfw_xkb_update_ime_state(_GLFWwindow *w, _GLFWXKBData *xkb, int which, int a, int b, int c, int d);
-void glfw_xkb_key_from_ime(KeyEvent *ev, GLFWbool handled_by_ime, GLFWbool failed);
+void glfw_xkb_key_from_ime(KeyEvent *ev, bool handled_by_ime, bool failed);
