@@ -5,25 +5,34 @@
  * Distributed under terms of the GPL3 license.
  */
 
+#include "loop-utils.h"
 #include "state.h"
 #include "threading.h"
 #include "screen.h"
 #include "fonts.h"
+#include "charsets.h"
 #include <termios.h>
 #include <unistd.h>
 #include <float.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <sys/wait.h>
-#include <signal.h>
 #include <sys/socket.h>
+#include <signal.h>
 extern PyTypeObject Screen_Type;
 
+#ifdef DEBUG_EVENT_LOOP
+#define EVDBG(...) log_event(__VA_ARGS__)
+#else
+#define EVDBG(...)
+#endif
+
 #define EXTRA_FDS 2
 #ifndef MSG_NOSIGNAL
 // Apple does not implement MSG_NOSIGNAL
 #define MSG_NOSIGNAL 0
 #endif
+#define USE_RENDER_FRAMES (global_state.has_render_frames && OPT(sync_to_monitor))
 
 static void (*parse_func)(Screen*, PyObject*, double);
 
@@ -44,6 +53,7 @@ typedef struct {
     int talk_fd, listen_fd;
     Message *messages;
     size_t messages_capacity, messages_count;
+    LoopData io_loop_data;
 } ChildMonitor;
 
 
@@ -75,7 +85,17 @@ static pthread_mutex_t children_lock;
 static bool kill_signal_received = false;
 static ChildMonitor *the_monitor = NULL;
 static uint8_t drain_buf[1024];
-static int signal_fds[2], wakeup_fds[2];
+
+
+typedef struct {
+    pid_t pid;
+    int status;
+} ReapedPID;
+
+static pid_t monitored_pids[256] = {0};
+static size_t monitored_pids_count = 0;
+static ReapedPID reaped_pids[arraysz(monitored_pids)] = {{0}};
+static size_t reaped_pids_count = 0;
 
 
 
@@ -97,39 +117,6 @@ set_maximum_wait(double val) {
     if (val >= 0 && (val < maximum_wait || maximum_wait < 0)) maximum_wait = val;
 }
 
-static void
-handle_signal(int sig_num) {
-    int save_err = errno;
-    unsigned char byte = (unsigned char)sig_num;
-    while(true) {
-        ssize_t ret = write(signal_fds[1], &byte, 1);
-        if (ret < 0 && errno == EINTR) continue;
-        break;
-    }
-    errno = save_err;
-}
-
-static inline bool
-self_pipe(int fds[2]) {
-#ifdef __APPLE__
-    int flags;
-    flags = pipe(fds);
-    if (flags != 0) return false;
-    for (int i = 0; i < 2; i++) {
-        flags = fcntl(fds[i], F_GETFD);
-        if (flags == -1) {  return false; }
-        if (fcntl(fds[i], F_SETFD, flags | FD_CLOEXEC) == -1) { return false; }
-        flags = fcntl(fds[i], F_GETFL);
-        if (flags == -1) { return false; }
-        if (fcntl(fds[i], F_SETFL, flags | O_NONBLOCK) == -1) { return false; }
-    }
-    return true;
-#else
-    return pipe2(fds, O_CLOEXEC | O_NONBLOCK) == 0;
-#endif
-}
-
-
 static PyObject *
 new(PyTypeObject *type, PyObject *args, PyObject UNUSED *kwds) {
     ChildMonitor *self;
@@ -143,13 +130,9 @@ new(PyTypeObject *type, PyObject *args, PyObject UNUSED *kwds) {
         PyErr_Format(PyExc_RuntimeError, "Failed to create children_lock mutex: %s", strerror(ret));
         return NULL;
     }
-    if (!self_pipe(wakeup_fds)) return PyErr_SetFromErrno(PyExc_OSError);
-    if (!self_pipe(signal_fds)) return PyErr_SetFromErrno(PyExc_OSError);
-    struct sigaction act = {.sa_handler=handle_signal};
-#define SA(which) { if (sigaction(which, &act, NULL) != 0) return PyErr_SetFromErrno(PyExc_OSError); if (siginterrupt(which, false) != 0) return PyErr_SetFromErrno(PyExc_OSError);}
-    SA(SIGINT); SA(SIGTERM); SA(SIGCHLD);
-#undef SA
     self = (ChildMonitor *)type->tp_alloc(type, 0);
+    if (!init_loop_data(&self->io_loop_data)) return PyErr_SetFromErrno(PyExc_OSError);
+    if (!install_signal_handlers(&self->io_loop_data)) return PyErr_SetFromErrno(PyExc_OSError);
     self->talk_fd = talk_fd;
     self->listen_fd = listen_fd;
     if (self == NULL) return PyErr_NoMemory();
@@ -159,7 +142,7 @@ new(PyTypeObject *type, PyObject *args, PyObject UNUSED *kwds) {
         parse_func = parse_worker_dump;
     } else parse_func = parse_worker;
     self->count = 0;
-    fds[0].fd = wakeup_fds[0]; fds[1].fd = signal_fds[0];
+    fds[0].fd = self->io_loop_data.wakeup_read_fd; fds[1].fd = self->io_loop_data.signal_read_fd;
     fds[0].events = POLLIN; fds[1].events = POLLIN;
     the_monitor = self;
 
@@ -180,22 +163,12 @@ dealloc(ChildMonitor* self) {
         add_queue_count--;
         FREE_CHILD(add_queue[add_queue_count]);
     }
-    close(wakeup_fds[0]);
-    close(wakeup_fds[1]);
-    close(signal_fds[0]);
-    close(signal_fds[1]);
+    free_loop_data(&self->io_loop_data);
 }
 
-void
-wakeup_io_loop(bool in_signal_handler) {
-    while(true) {
-        ssize_t ret = write(wakeup_fds[1], "w", 1);
-        if (ret < 0) {
-            if (errno == EINTR) continue;
-            if (!in_signal_handler) perror("Failed to write to wakeup fd with error");
-        }
-        break;
-    }
+static void
+wakeup_io_loop(ChildMonitor *self, bool in_signal_handler) {
+    wakeup_loop(&self->io_loop_data, in_signal_handler);
 }
 
 static void* io_loop(void *data);
@@ -218,11 +191,10 @@ start(PyObject *s, PyObject *a UNUSED) {
     Py_RETURN_NONE;
 }
 
-
 static PyObject *
-wakeup(PYNOARG) {
+wakeup(ChildMonitor *self, PyObject *args UNUSED) {
 #define wakeup_doc "wakeup() -> wakeup the ChildMonitor I/O thread, forcing it to exit from poll() if it is waiting there."
-    wakeup_io_loop(false);
+    wakeup_io_loop(self, false);
     Py_RETURN_NONE;
 }
 
@@ -241,7 +213,7 @@ add_child(ChildMonitor *self, PyObject *args) {
     INCREF_CHILD(add_queue[add_queue_count]);
     add_queue_count++;
     children_mutex(unlock);
-    wakeup_io_loop(false);
+    wakeup_io_loop(self, false);
     Py_RETURN_NONE;
 }
 
@@ -288,7 +260,7 @@ schedule_write_to_child(unsigned long id, unsigned int num, ...) {
                 screen->write_buf = PyMem_RawRealloc(screen->write_buf, screen->write_buf_sz);
                 if (screen->write_buf == NULL) { fatal("Out of memory."); }
             }
-            if (screen->write_buf_used) wakeup_io_loop(false);
+            if (screen->write_buf_used) wakeup_io_loop(self, false);
             screen_mutex(unlock, write);
             break;
         }
@@ -310,12 +282,9 @@ needs_write(ChildMonitor UNUSED *self, PyObject *args) {
 static PyObject *
 shutdown_monitor(ChildMonitor *self, PyObject *a UNUSED) {
 #define shutdown_monitor_doc "shutdown_monitor() -> Shutdown the monitor loop."
-    signal(SIGINT, SIG_DFL);
-    signal(SIGTERM, SIG_DFL);
-    signal(SIGCHLD, SIG_DFL);
     self->shutting_down = true;
     wakeup_talk_loop(false);
-    wakeup_io_loop(false);
+    wakeup_io_loop(self, false);
     int ret = pthread_join(self->io_thread, NULL);
     if (ret != 0) return PyErr_Format(PyExc_OSError, "Failed to join() I/O thread with error: %s", strerror(ret));
     if (talk_thread_started) {
@@ -326,15 +295,17 @@ shutdown_monitor(ChildMonitor *self, PyObject *a UNUSED) {
     Py_RETURN_NONE;
 }
 
-static inline void
+static inline bool
 do_parse(ChildMonitor *self, Screen *screen, double now) {
+    bool input_read = false;
     screen_mutex(lock, read);
     if (screen->read_buf_sz || screen->pending_mode.used) {
         double time_since_new_input = now - screen->new_input_at;
         if (time_since_new_input >= OPT(input_delay)) {
             bool read_buf_full = screen->read_buf_sz >= READ_BUF_SZ;
+            input_read = true;
             parse_func(screen, self->dump_callback, now);
-            if (read_buf_full) wakeup_io_loop(false);  // Ensure the read fd has POLLIN set
+            if (read_buf_full) wakeup_io_loop(self, false);  // Ensure the read fd has POLLIN set
             screen->new_input_at = 0;
             if (screen->pending_mode.activated_at) {
                 double time_since_pending = MAX(0, now - screen->pending_mode.activated_at);
@@ -343,12 +314,14 @@ do_parse(ChildMonitor *self, Screen *screen, double now) {
         } else set_maximum_wait(OPT(input_delay) - time_since_new_input);
     }
     screen_mutex(unlock, read);
+    return input_read;
 }
 
-static void
+static bool
 parse_input(ChildMonitor *self) {
     // Parse all available input that was read in the I/O thread.
     size_t count = 0, remove_count = 0;
+    bool input_read = false;
     double now = monotonic();
     PyObject *msg = NULL;
     children_mutex(lock);
@@ -403,10 +376,11 @@ parse_input(ChildMonitor *self) {
 
     for (size_t i = 0; i < count; i++) {
         if (!scratch[i].needs_removal) {
-            do_parse(self, scratch[i].screen, now);
+            if (do_parse(self, scratch[i].screen, now)) input_read = true;
         }
         DECREF_CHILD(scratch[i]);
     }
+    return input_read;
 }
 
 static inline void
@@ -419,7 +393,7 @@ mark_child_for_close(ChildMonitor *self, id_type window_id) {
         }
     }
     children_mutex(unlock);
-    wakeup_io_loop(false);
+    wakeup_io_loop(self, false);
 }
 
 
@@ -508,9 +482,7 @@ pyset_iutf8(ChildMonitor *self, PyObject *args) {
 #undef INCREF_CHILD
 #undef DECREF_CHILD
 
-static double last_render_at = -DBL_MAX;
-
-extern void cocoa_update_title(PyObject*);
+extern void cocoa_update_menu_bar_title(PyObject*);
 
 static inline void
 collect_cursor_info(CursorRenderInfo *ans, Window *w, double now, OSWindow *os_window) {
@@ -546,7 +518,7 @@ update_window_title(Window *w, OSWindow *os_window) {
         Py_INCREF(os_window->window_title);
         set_os_window_title(os_window, PyUnicode_AsUTF8(w->title));
 #ifdef __APPLE__
-        if (os_window->is_focused) cocoa_update_title(w->title);
+        if (os_window->is_focused && OPT(macos_show_window_title_in_menubar)) cocoa_update_menu_bar_title(w->title);
 #endif
         return true;
     }
@@ -558,7 +530,7 @@ prepare_to_render_os_window(OSWindow *os_window, double now, unsigned int *activ
 #define TD os_window->tab_bar_render_data
     bool needs_render = os_window->needs_render;
     os_window->needs_render = false;
-    if (TD.screen && os_window->num_tabs > 1) {
+    if (TD.screen && os_window->num_tabs >= OPT(tab_bar_min_tabs)) {
         if (!os_window->tab_bar_data_updated) {
             call_boss(update_tab_bar_data, "K", os_window->id);
             os_window->tab_bar_data_updated = true;
@@ -606,13 +578,22 @@ render_os_window(OSWindow *os_window, double now, unsigned int active_window_id,
     if (os_window->clear_count++ < 3) blank_os_window(os_window);
     Tab *tab = os_window->tabs + os_window->active_tab;
     BorderRects *br = &tab->border_rects;
-    draw_borders(br->vao_idx, br->num_border_rects, br->rect_buf, br->is_dirty, os_window->viewport_width, os_window->viewport_height, active_window_bg, num_visible_windows, os_window);
-    if (TD.screen && os_window->num_tabs > 1) draw_cells(TD.vao_idx, 0, TD.xstart, TD.ystart, TD.dx, TD.dy, TD.screen, os_window, true, false);
+    bool static_live_resize_in_progress = os_window->live_resize.in_progress && OPT(resize_draw_strategy) == RESIZE_DRAW_STATIC;
+    float x_ratio = 1, y_ratio = 1;
+    if (static_live_resize_in_progress) {
+        x_ratio = os_window->viewport_width / (double) os_window->live_resize.width;
+        y_ratio = os_window->viewport_height / (double) os_window->live_resize.height;
+    }
+    if (!static_live_resize_in_progress) {
+        draw_borders(br->vao_idx, br->num_border_rects, br->rect_buf, br->is_dirty, os_window->viewport_width, os_window->viewport_height, active_window_bg, num_visible_windows, os_window);
+        br->is_dirty = false;
+    }
+    if (TD.screen && os_window->num_tabs >= OPT(tab_bar_min_tabs)) draw_cells(TD.vao_idx, 0, TD.xstart, TD.ystart, TD.dx * x_ratio, TD.dy * y_ratio, TD.screen, os_window, true, false);
     for (unsigned int i = 0; i < tab->num_windows; i++) {
         Window *w = tab->windows + i;
         if (w->visible && WD.screen) {
             bool is_active_window = i == tab->active_window;
-            draw_cells(WD.vao_idx, WD.gvao_idx, WD.xstart, WD.ystart, WD.dx, WD.dy, WD.screen, os_window, is_active_window, true);
+            draw_cells(WD.vao_idx, WD.gvao_idx, WD.xstart, WD.ystart, WD.dx * x_ratio, WD.dy * y_ratio, WD.screen, os_window, is_active_window, true);
             if (WD.screen->start_visual_bell_at != 0) {
                 double bell_left = global_state.opts.visual_bell_duration - (now - WD.screen->start_visual_bell_at);
                 set_maximum_wait(bell_left);
@@ -621,32 +602,73 @@ render_os_window(OSWindow *os_window, double now, unsigned int active_window_id,
         }
     }
     swap_window_buffers(os_window);
-    br->is_dirty = false;
     os_window->last_active_tab = os_window->active_tab; os_window->last_num_tabs = os_window->num_tabs; os_window->last_active_window_id = active_window_id;
     os_window->focused_at_last_render = os_window->is_focused;
     os_window->is_damaged = false;
-    if (global_state.is_wayland && OPT(sync_to_monitor)) wayland_request_frame_render(os_window);
+    if (USE_RENDER_FRAMES) request_frame_render(os_window);
 #undef WD
 #undef TD
 }
 
 static inline void
-render(double now) {
+update_os_window_title(OSWindow *os_window) {
+    Tab *tab = os_window->tabs + os_window->active_tab;
+    if (tab->num_windows) {
+        Window *w = tab->windows + tab->active_window;
+        update_window_title(w, os_window);
+    }
+}
+
+static void
+draw_resizing_text(OSWindow *w) {
+    char text[32] = {0};
+    unsigned int width = w->live_resize.width, height = w->live_resize.height;
+    snprintf(text, sizeof(text), "%u x %u cells", width / w->fonts_data->cell_width, height / w->fonts_data->cell_height);
+    StringCanvas rendered = render_simple_text(w->fonts_data, text);
+    if (rendered.canvas) {
+        draw_centered_alpha_mask(w->gvao_idx, width, height, rendered.width, rendered.height, rendered.canvas);
+        free(rendered.canvas);
+    }
+}
+
+static inline bool
+no_render_frame_received_recently(OSWindow *w, double now, double max_wait) {
+    bool ans = now - w->last_render_frame_received_at > max_wait;
+    if (ans) log_error("No render frame received in %f seconds, re-requesting at: %f", max_wait, now);
+    return ans;
+}
+
+static inline void
+render(double now, bool input_read) {
+    EVDBG("input_read: %d", input_read);
+    static double last_render_at = -DBL_MAX;
     double time_since_last_render = now - last_render_at;
-    if (time_since_last_render < OPT(repaint_delay)) {
+    if (!input_read && time_since_last_render < OPT(repaint_delay)) {
         set_maximum_wait(OPT(repaint_delay) - time_since_last_render);
         return;
     }
 
     for (size_t i = 0; i < global_state.num_os_windows; i++) {
         OSWindow *w = global_state.os_windows + i;
-        if (!w->num_tabs || !should_os_window_be_rendered(w)) continue;
-        if (global_state.is_wayland && w->wayland_render_state != RENDER_FRAME_READY && OPT(sync_to_monitor)) {
-            if (w->wayland_render_state == RENDER_FRAME_NOT_REQUESTED) wayland_request_frame_render(w);
+        if (!w->num_tabs) continue;
+        if (!should_os_window_be_rendered(w)) {
+            update_os_window_title(w);
+            continue;
+        }
+        if (USE_RENDER_FRAMES && w->render_state != RENDER_FRAME_READY) {
+            if (w->render_state == RENDER_FRAME_NOT_REQUESTED || no_render_frame_received_recently(w, now, 0.25)) request_frame_render(w);
             continue;
         }
-        bool needs_render = w->is_damaged;
         make_os_window_context_current(w);
+        if (w->live_resize.in_progress && OPT(resize_draw_strategy) >= RESIZE_DRAW_BLANK) {
+            blank_os_window(w);
+            if (OPT(resize_draw_strategy) == RESIZE_DRAW_SIZE) draw_resizing_text(w);
+            swap_window_buffers(w);
+            if (USE_RENDER_FRAMES) request_frame_render(w);
+            continue;
+        }
+        if (w->live_resize.in_progress && OPT(resize_draw_strategy) == RESIZE_DRAW_STATIC) blank_os_window(w);
+        bool needs_render = w->is_damaged || w->live_resize.in_progress;
         if (w->viewport_size_dirty) {
             w->clear_count = 0;
             update_surface_size(w->viewport_width, w->viewport_height, w->offscreen_texture_id);
@@ -669,7 +691,7 @@ typedef struct { int fd; uint8_t *buf; size_t sz; } ThreadWriteData;
 
 static inline ThreadWriteData*
 alloc_twd(size_t sz) {
-    ThreadWriteData *data = malloc(sizeof(ThreadWriteData));
+    ThreadWriteData *data = calloc(1, sizeof(ThreadWriteData));
     if (data != NULL) {
         data->sz = sz;
         data->buf = malloc(sz);
@@ -684,15 +706,58 @@ free_twd(ThreadWriteData *x) {
     free(x);
 }
 
+static PyObject*
+monitor_pid(PyObject *self UNUSED, PyObject *args) {
+    long pid;
+    bool ok = true;
+    if (!PyArg_ParseTuple(args, "l", &pid)) return NULL;
+    children_mutex(lock);
+    if (monitored_pids_count >= arraysz(monitored_pids)) {
+        PyErr_SetString(PyExc_RuntimeError, "Too many monitored pids");
+        ok = false;
+    } else {
+        monitored_pids[monitored_pids_count++] = pid;
+    }
+    children_mutex(unlock);
+    if (!ok) return NULL;
+    Py_RETURN_NONE;
+}
+
+static inline void
+report_reaped_pids(void) {
+    children_mutex(lock);
+    if (reaped_pids_count) {
+        for (size_t i = 0; i < reaped_pids_count; i++) {
+            call_boss(on_monitored_pid_death, "ii", (int)reaped_pids[i].pid, reaped_pids[i].status);
+        }
+        reaped_pids_count = 0;
+    }
+    children_mutex(unlock);
+}
+
 static void*
 thread_write(void *x) {
     ThreadWriteData *data = (ThreadWriteData*)x;
     set_thread_name("KittyWriteStdin");
-    FILE *f = fdopen(data->fd, "w");
-    if (fwrite(data->buf, 1, data->sz, f) != data->sz) {
-        log_error("Failed to write all data");
+    int flags = fcntl(data->fd, F_GETFL, 0);
+    if (flags == -1) { free_twd(data); return 0; }
+    flags &= ~O_NONBLOCK;
+    fcntl(data->fd, F_SETFL, flags);
+    size_t pos = 0;
+    while (pos < data->sz) {
+        errno = 0;
+        ssize_t nbytes = write(data->fd, data->buf + pos, data->sz - pos);
+        if (nbytes < 0) {
+            if (errno == EAGAIN || errno == EINTR) continue;
+            break;
+        }
+        if (nbytes == 0) break;
+        pos += nbytes;
     }
-    fclose(f);
+    if (pos < data->sz) {
+        log_error("Failed to write all data to STDIN of child process with error: %s", strerror(errno));
+    }
+    safe_close(data->fd);
     free_twd(data);
     return 0;
 }
@@ -709,38 +774,83 @@ cm_thread_write(PyObject UNUSED *self, PyObject *args) {
     data->fd = fd;
     memcpy(data->buf, buf, data->sz);
     int ret = pthread_create(&thread, NULL, thread_write, data);
-    if (ret != 0) { free_twd(data); return PyErr_SetFromErrno(PyExc_OSError); }
+    if (ret != 0) { safe_close(fd); free_twd(data); return PyErr_SetFromErrno(PyExc_OSError); }
+    pthread_detach(thread);
     Py_RETURN_NONE;
 }
 
-static inline void
-wait_for_events() {
-    event_loop_wait(maximum_wait);
-    maximum_wait = -1;
+static void
+python_timer_callback(id_type timer_id, void *data) {
+    PyObject *callback = (PyObject*)data;
+    unsigned long long id = timer_id;
+    PyObject *ret = PyObject_CallFunction(callback, "K", id);
+    if (ret == NULL) PyErr_Print();
+    else Py_DECREF(ret);
 }
 
+static void
+python_timer_cleanup(id_type timer_id UNUSED, void *data) {
+    if (data) Py_DECREF((PyObject*)data);
+}
+
+static PyObject*
+add_python_timer(PyObject *self UNUSED, PyObject *args) {
+    PyObject *callback;
+    double interval;
+    int repeats = 1;
+    if (!PyArg_ParseTuple(args, "Od|p", &callback, &interval, &repeats)) return NULL;
+    unsigned long long timer_id = add_main_loop_timer(interval, repeats ? true: false, python_timer_callback, callback, python_timer_cleanup);
+    Py_INCREF(callback);
+    return Py_BuildValue("K", timer_id);
+}
+
+static PyObject*
+remove_python_timer(PyObject *self UNUSED, PyObject *args) {
+    unsigned long long timer_id;
+    if (!PyArg_ParseTuple(args, "K", &timer_id)) return NULL;
+    remove_main_loop_timer(timer_id);
+    Py_RETURN_NONE;
+}
+
+
 static inline void
 process_pending_resizes(double now) {
     global_state.has_pending_resizes = false;
     for (size_t i = 0; i < global_state.num_os_windows; i++) {
         OSWindow *w = global_state.os_windows + i;
-        if (w->has_pending_resizes) {
-            if (now - w->last_resize_event_at >= RESIZE_DEBOUNCE_TIME) update_os_window_viewport(w, true);
-            else {
-                global_state.has_pending_resizes = true;
-                set_maximum_wait(RESIZE_DEBOUNCE_TIME - now + w->last_resize_event_at);
+        if (w->live_resize.in_progress) {
+            bool update_viewport = false;
+            if (w->live_resize.from_os_notification) {
+                if (w->live_resize.os_says_resize_complete || (now - w->live_resize.last_resize_event_at) > 1) update_viewport = true;
+            } else {
+                double debounce_time = OPT(resize_debounce_time);
+                // if more than one resize event has occurred, wait at least 0.2 secs
+                // before repainting, to avoid rapid transitions between the cells banner
+                // and the normal screen
+                if (w->live_resize.num_of_resize_events > 1 && OPT(resize_draw_strategy) == RESIZE_DRAW_SIZE) debounce_time = MAX(0.2, debounce_time);
+                if (now - w->live_resize.last_resize_event_at >= debounce_time) update_viewport = true;
+                else {
+                    global_state.has_pending_resizes = true;
+                    set_maximum_wait(OPT(resize_debounce_time) - now + w->live_resize.last_resize_event_at);
+                }
+            }
+            if (update_viewport) {
+                static const LiveResizeInfo empty = {0};
+                update_os_window_viewport(w, true);
+                w->live_resize = empty;
             }
         }
     }
 }
 
 static inline void
-close_all_windows() {
+close_all_windows(void) {
     for (size_t w = 0; w < global_state.num_os_windows; w++) mark_os_window_for_close(&global_state.os_windows[w], true);
 }
 
 static inline bool
 process_pending_closes(ChildMonitor *self) {
+    global_state.has_pending_closes = false;
     bool has_open_windows = false;
     for (size_t w = global_state.num_os_windows; w > 0; w--) {
         OSWindow *os_window = global_state.os_windows + w - 1;
@@ -767,43 +877,84 @@ process_pending_closes(ChildMonitor *self) {
 // via the mouse causes a crash because of the way autorelease pools work in
 // glfw/cocoa. So we use a flag instead.
 static unsigned int cocoa_pending_actions = 0;
+static char *cocoa_pending_actions_wd = NULL;
 
 void
-set_cocoa_pending_action(CocoaPendingAction action) {
+set_cocoa_pending_action(CocoaPendingAction action, const char *wd) {
+    if (wd) {
+        if (cocoa_pending_actions_wd) free(cocoa_pending_actions_wd);
+        cocoa_pending_actions_wd = strdup(wd);
+    }
     cocoa_pending_actions |= action;
     // The main loop may be blocking on the event queue, if e.g. unfocused.
     // Unjam it so the pending action is processed right now.
-    unjam_event_loop();
+    wakeup_main_loop();
 }
 #endif
 
-static PyObject*
-main_loop(ChildMonitor *self, PyObject *a UNUSED) {
-#define main_loop_doc "The main thread loop"
-    bool has_open_windows = true;
+static void process_global_state(void *data);
 
-    while (has_open_windows) {
-        double now = monotonic();
-        if (global_state.has_pending_resizes) process_pending_resizes(now);
-        render(now);
-        wait_for_events();
+static void
+do_state_check(id_type timer_id UNUSED, void *data) {
+    EVDBG("State check timer fired");
+    process_global_state(data);
+}
+
+static id_type state_check_timer = 0;
+
+static void
+process_global_state(void *data) {
+    EVDBG("Processing global state");
+    ChildMonitor *self = data;
+    maximum_wait = -1;
+    bool state_check_timer_enabled = false;
+
+    double now = monotonic();
+    if (global_state.has_pending_resizes) process_pending_resizes(now);
+    bool input_read = parse_input(self);
+    render(now, input_read);
 #ifdef __APPLE__
         if (cocoa_pending_actions) {
             if (cocoa_pending_actions & PREFERENCES_WINDOW) { call_boss(edit_config_file, NULL); }
             if (cocoa_pending_actions & NEW_OS_WINDOW) { call_boss(new_os_window, NULL); }
+            if (cocoa_pending_actions_wd) {
+                if (cocoa_pending_actions & NEW_OS_WINDOW_WITH_WD) { call_boss(new_os_window_with_wd, "s", cocoa_pending_actions_wd); }
+                if (cocoa_pending_actions & NEW_TAB_WITH_WD) { call_boss(new_tab_with_wd, "s", cocoa_pending_actions_wd); }
+                free(cocoa_pending_actions_wd);
+                cocoa_pending_actions_wd = NULL;
+            }
             cocoa_pending_actions = 0;
         }
 #endif
-        parse_input(self);
-        if (global_state.terminate) {
-            global_state.terminate = false;
-            close_all_windows();
+    if (global_state.terminate) {
+        global_state.terminate = false;
+        close_all_windows();
 #ifdef __APPLE__
-            request_application_quit();
+        request_application_quit();
 #endif
-        }
-        has_open_windows = process_pending_closes(self);
     }
+    report_reaped_pids();
+    bool has_open_windows = true;
+    if (global_state.has_pending_closes) has_open_windows = process_pending_closes(self);
+    if (has_open_windows) {
+        if (maximum_wait >= 0) {
+            if (maximum_wait == 0) request_tick_callback();
+            else state_check_timer_enabled = true;
+        }
+    } else {
+        stop_main_loop();
+    }
+    update_main_loop_timer(state_check_timer, MAX(0, maximum_wait), state_check_timer_enabled);
+}
+
+static PyObject*
+main_loop(ChildMonitor *self, PyObject *a UNUSED) {
+#define main_loop_doc "The main thread loop"
+    state_check_timer = add_main_loop_timer(1000, true, do_state_check, self, NULL);
+    run_main_loop(process_global_state, self);
+#ifdef __APPLE__
+    if (cocoa_pending_actions_wd) { free(cocoa_pending_actions_wd); cocoa_pending_actions_wd = NULL; }
+#endif
     if (PyErr_Occurred()) return NULL;
     Py_RETURN_NONE;
 }
@@ -839,7 +990,7 @@ hangup(pid_t pid) {
 
 static inline void
 cleanup_child(ssize_t i) {
-    close(children[i].fd);
+    safe_close(children[i].fd);
     hangup(children[i].pid);
 }
 
@@ -908,35 +1059,28 @@ drain_fd(int fd) {
         ssize_t len = read(fd, drain_buf, sizeof(drain_buf));
         if (len < 0) {
             if (errno == EINTR) continue;
-            if (errno != EIO) perror("Call to read() from drain fd failed");
             break;
         }
+        if (len > 0) continue;
         break;
     }
 }
 
-static inline void
-read_signals(int fd, bool *kill_signal, bool *child_died) {
-    static char buf[256];
-    while(true) {
-        ssize_t len = read(fd, buf, sizeof(buf));
-        if (len < 0) {
-            if (errno == EINTR) continue;
-            if (errno != EIO) perror("Call to read() from read_signals() failed");
+typedef struct { bool kill_signal, child_died; } SignalSet;
+
+static void
+handle_signal(int signum, void *data) {
+    SignalSet *ss = data;
+    switch(signum) {
+        case SIGINT:
+        case SIGTERM:
+            ss->kill_signal = true;
+            break;
+        case SIGCHLD:
+            ss->child_died = true;
+            break;
+        default:
             break;
-        }
-        for (ssize_t i = 0; i < len; i++) {
-            switch(buf[i]) {
-                case SIGCHLD:
-                    *child_died = true; break;
-                case SIGINT:
-                case SIGTERM:
-                    *kill_signal = true; break;
-                default:
-                    break;
-            }
-        }
-        break;
     }
 }
 
@@ -952,6 +1096,21 @@ mark_child_for_removal(ChildMonitor *self, pid_t pid) {
     children_mutex(unlock);
 }
 
+static inline void
+mark_monitored_pids(pid_t pid, int status) {
+    children_mutex(lock);
+    for (ssize_t i = monitored_pids_count - 1; i >= 0; i--) {
+        if (pid == monitored_pids[i]) {
+            if (reaped_pids_count < arraysz(reaped_pids)) {
+                reaped_pids[reaped_pids_count].status = status;
+                reaped_pids[reaped_pids_count++].pid = pid;
+            }
+            remove_i_from_array(monitored_pids, (size_t)i, monitored_pids_count);
+        }
+    }
+    children_mutex(unlock);
+}
+
 static inline void
 reap_children(ChildMonitor *self, bool enable_close_on_child_death) {
     int status;
@@ -963,6 +1122,7 @@ reap_children(ChildMonitor *self, bool enable_close_on_child_death) {
             if (errno != EINTR) break;
         } else if (pid > 0) {
             if (enable_close_on_child_death) mark_child_for_removal(self, pid);
+            mark_monitored_pids(pid, status);
         } else break;
     }
 }
@@ -1030,11 +1190,11 @@ io_loop(void *data) {
         if (ret > 0) {
             if (fds[0].revents && POLLIN) drain_fd(fds[0].fd); // wakeup
             if (fds[1].revents && POLLIN) {
+                SignalSet ss = {0};
                 data_received = true;
-                bool kill_signal = false, child_died = false;
-                read_signals(fds[1].fd, &kill_signal, &child_died);
-                if (kill_signal) { children_mutex(lock); kill_signal_received = true; children_mutex(unlock); }
-                if (child_died) reap_children(self, OPT(close_on_child_death));
+                read_signals(fds[1].fd, handle_signal, &ss);
+                if (ss.kill_signal) { children_mutex(lock); kill_signal_received = true; children_mutex(unlock); }
+                if (ss.child_died) reap_children(self, OPT(close_on_child_death));
             }
             for (i = 0; i < self->count; i++) {
                 if (fds[EXTRA_FDS + i].revents & (POLLIN | POLLHUP)) {
@@ -1114,14 +1274,14 @@ typedef struct {
     PeerReadData *reads;
     PeerWriteData *writes;
     PeerWriteData *queued_writes;
-    int wakeup_fds[2];
+    LoopData loop_data;
     pthread_mutex_t peer_lock;
 } TalkData;
 
 static TalkData talk_data = {0};
 typedef struct pollfd PollFD;
 #define PEER_LIMIT 256
-#define nuke_socket(s) { shutdown(s, SHUT_RDWR); close(s); }
+#define nuke_socket(s) { shutdown(s, SHUT_RDWR); safe_close(s); }
 
 static inline bool
 accept_peer(int listen_fd, bool shutting_down) {
@@ -1154,7 +1314,7 @@ read_from_peer(ChildMonitor *self, int s) {
         if (rd->fd == s) {
             if (rd->used >= rd->capacity) {
                 if (rd->capacity >= 1024 * 1024) failed("Ignoring too large message from peer");
-                rd->capacity = MAX(8192, rd->capacity * 2);
+                rd->capacity = MAX(8192u, rd->capacity * 2);
                 rd->data = realloc(rd->data, rd->capacity);
                 if (!rd->data) failed("Out of memory");
             }
@@ -1218,7 +1378,7 @@ remove_poll_fd(int fd) {
 }
 
 static inline void
-prune_finished_reads() {
+prune_finished_reads(void) {
     if (!talk_data.num_reads) return;
     for (ssize_t i = talk_data.num_reads - 1; i >= 0; i--) {
         PeerReadData *rd = talk_data.reads + i;
@@ -1236,13 +1396,13 @@ prune_finished_reads() {
 }
 
 static inline void
-prune_finished_writes() {
+prune_finished_writes(void) {
     if (!talk_data.num_writes) return;
     for (ssize_t i = talk_data.num_writes - 1; i >= 0; i--) {
         PeerWriteData *wd = talk_data.writes + i;
         if (wd->finished) {
             remove_poll_fd(wd->fd);
-            shutdown(wd->fd, SHUT_WR); close(wd->fd);
+            shutdown(wd->fd, SHUT_WR); safe_close(wd->fd);
             free(wd->data);
             ssize_t num_to_right = talk_data.num_writes - 1 - i;
             if (num_to_right > 0) memmove(talk_data.writes + i, talk_data.writes + i + 1, num_to_right * sizeof(PeerWriteData));
@@ -1254,19 +1414,11 @@ prune_finished_writes() {
 
 static void
 wakeup_talk_loop(bool in_signal_handler) {
-    if (talk_data.wakeup_fds[1] <= 0) return;
-    while(true) {
-        ssize_t ret = write(talk_data.wakeup_fds[1], "w", 1);
-        if (ret < 0) {
-            if (errno == EINTR) continue;
-            if (!in_signal_handler) perror("Failed to write to talk wakeup fd with error");
-        }
-        break;
-    }
+    wakeup_loop(&talk_data.loop_data, in_signal_handler);
 }
 
 static inline void
-move_queued_writes() {
+move_queued_writes(void) {
     while (talk_data.num_queued_writes) {
         PeerWriteData *src = talk_data.queued_writes + --talk_data.num_queued_writes;
         size_t fd_idx = talk_data.num_listen_fds + talk_data.num_talk_fds;
@@ -1291,7 +1443,7 @@ talk_loop(void *data) {
     ChildMonitor *self = (ChildMonitor*)data;
     set_thread_name("KittyPeerMon");
     if ((pthread_mutex_init(&talk_data.peer_lock, NULL)) != 0) { perror("Failed to create peer mutex"); return 0; }
-    if (!self_pipe(talk_data.wakeup_fds)) { perror("Failed to create wakeup fds for talk thread"); return 0; }
+    if (!init_loop_data(&talk_data.loop_data)) { log_error("Failed to create wakeup fd for talk thread with error: %s", strerror(errno)); }
     ensure_space_for(&talk_data, fds, PollFD, 8, fds_capacity, 8, false);
 #define add_listener(which) \
     if (self->which > -1) { \
@@ -1299,7 +1451,7 @@ talk_loop(void *data) {
     }
     add_listener(talk_fd); add_listener(listen_fd);
 #undef add_listener
-    talk_data.fds[talk_data.num_listen_fds].fd = talk_data.wakeup_fds[0]; talk_data.fds[talk_data.num_listen_fds++].events = POLLIN;
+    talk_data.fds[talk_data.num_listen_fds].fd = talk_data.loop_data.wakeup_read_fd; talk_data.fds[talk_data.num_listen_fds++].events = POLLIN;
 
     while (LIKELY(!self->shutting_down)) {
         for (size_t i = 0; i < talk_data.num_listen_fds + talk_data.num_talk_fds; i++) { talk_data.fds[i].revents = 0; }
@@ -1322,7 +1474,7 @@ talk_loop(void *data) {
         } else if (ret < 0) { if (errno != EAGAIN && errno != EINTR) perror("poll() on talk fds failed"); }
     }
 end:
-    close(talk_data.wakeup_fds[0]); close(talk_data.wakeup_fds[1]);
+    free_loop_data(&talk_data.loop_data);
     free(talk_data.fds); free(talk_data.reads); free(talk_data.writes); free(talk_data.queued_writes);
     return 0;
 }
@@ -1348,8 +1500,8 @@ add_peer_writer(int fd, const char* msg, size_t msg_sz) {
 
 static void
 send_response(int fd, const char *msg, size_t msg_sz) {
-    if (msg == NULL) { shutdown(fd, SHUT_WR); close(fd); return; }
-    if (!add_peer_writer(fd, msg, msg_sz)) { shutdown(fd, SHUT_WR); close(fd); }
+    if (msg == NULL) { shutdown(fd, SHUT_WR); safe_close(fd); return; }
+    if (!add_peer_writer(fd, msg, msg_sz)) { shutdown(fd, SHUT_WR); safe_close(fd); }
     else wakeup_talk_loop(false);
 }
 
@@ -1365,7 +1517,6 @@ static PyMethodDef methods[] = {
     METHOD(main_loop, METH_NOARGS)
     METHOD(mark_for_close, METH_VARARGS)
     METHOD(resize_pty, METH_VARARGS)
-    {"set_iutf8", (PyCFunction)pyset_iutf8, METH_VARARGS, ""},
     {NULL}  /* Sentinel */
 };
 
@@ -1381,15 +1532,23 @@ PyTypeObject ChildMonitor_Type = {
     .tp_new = new,
 };
 
+
+
 static PyObject*
-safe_pipe(PYNOARG) {
+safe_pipe(PyObject *self UNUSED, PyObject *args) {
+    int nonblock = 1;
+    if (!PyArg_ParseTuple(args, "|p", &nonblock)) return NULL;
     int fds[2] = {0};
-    if (!self_pipe(fds)) return PyErr_SetFromErrno(PyExc_OSError);
+    if (!self_pipe(fds, nonblock)) return PyErr_SetFromErrno(PyExc_OSError);
     return Py_BuildValue("ii", fds[0], fds[1]);
 }
 
 static PyMethodDef module_methods[] = {
-    METHODB(safe_pipe, METH_NOARGS),
+    METHODB(safe_pipe, METH_VARARGS),
+    {"add_timer", (PyCFunction)add_python_timer, METH_VARARGS, ""},
+    {"remove_timer", (PyCFunction)remove_python_timer, METH_VARARGS, ""},
+    METHODB(monitor_pid, METH_VARARGS),
+    {"set_iutf8_winid", (PyCFunction)pyset_iutf8, METH_VARARGS, ""},
     {NULL}  /* Sentinel */
 };
 
