@@ -13,6 +13,10 @@
 #include <hb.h>
 #pragma GCC diagnostic pop
 
+typedef struct {
+    uint8_t *canvas;
+    size_t width, height;
+} StringCanvas;
 
 // API that font backends need to implement
 typedef uint16_t glyph_index;
@@ -30,9 +34,11 @@ PyObject* face_from_descriptor(PyObject*, FONTS_DATA_HANDLE);
 
 void sprite_tracker_current_layout(FONTS_DATA_HANDLE data, unsigned int *x, unsigned int *y, unsigned int *z);
 void render_alpha_mask(uint8_t *alpha_mask, pixel* dest, Region *src_rect, Region *dest_rect, size_t src_stride, size_t dest_stride);
-void render_line(FONTS_DATA_HANDLE, Line *line);
+void render_line(FONTS_DATA_HANDLE, Line *line, index_type lnum, Cursor *cursor, DisableLigature);
 void sprite_tracker_set_limits(size_t max_texture_size, size_t max_array_len);
 typedef void (*free_extra_data_func)(void*);
+StringCanvas render_simple_text_impl(PyObject *s, const char *text, unsigned int baseline);
+StringCanvas render_simple_text(FONTS_DATA_HANDLE fg_, const char *text);
 
 static inline void
 right_shift_canvas(pixel *canvas, size_t width, size_t height, size_t amt) {
@@ -40,6 +46,6 @@ right_shift_canvas(pixel *canvas, size_t width, size_t height, size_t amt) {
     size_t r;
     for (r = 0, src = canvas; r < height; r++, src += width) {
         memmove(src + amt, src, sizeof(pixel) * (width - amt));
-        memset(src, 0, sizeof(pixel) * amt);
+        zero_at_ptr_count(src, amt);
     }
 }
