@@ -10,14 +10,12 @@
 
 GlobalState global_state = {{0}};
 
-#define REMOVER(array, qid, count, structure, destroy, capacity) { \
+#define REMOVER(array, qid, count, destroy, capacity) { \
     for (size_t i = 0; i < count; i++) { \
         if (array[i].id == qid) { \
             destroy(array + i); \
-            memset(array + i, 0, sizeof(structure)); \
-            size_t num_to_right = count - 1 - i; \
-            if (num_to_right) memmove(array + i, array + i + 1, num_to_right * sizeof(structure)); \
-            (count)--; \
+            zero_at_i(array, i); \
+            remove_i_from_array(array, i, count); \
             break; \
         } \
     }}
@@ -78,9 +76,10 @@ add_os_window() {
     WITH_OS_WINDOW_REFS
     ensure_space_for(&global_state, os_windows, OSWindow, global_state.num_os_windows + 1, capacity, 1, true);
     OSWindow *ans = global_state.os_windows + global_state.num_os_windows++;
-    memset(ans, 0, sizeof(OSWindow));
+    zero_at_ptr(ans);
     ans->id = ++global_state.os_window_id_counter;
     ans->tab_bar_render_data.vao_idx = create_cell_vao();
+    ans->gvao_idx = create_graphics_vao();
     ans->background_opacity = OPT(background_opacity);
     ans->font_sz_in_pts = global_state.font_sz_in_pts;
     END_WITH_OS_WINDOW_REFS
@@ -92,7 +91,7 @@ add_tab(id_type os_window_id) {
     WITH_OS_WINDOW(os_window_id)
         make_os_window_context_current(os_window);
         ensure_space_for(os_window, tabs, Tab, os_window->num_tabs + 1, capacity, 1, true);
-        memset(os_window->tabs + os_window->num_tabs, 0, sizeof(Tab));
+        zero_at_i(os_window->tabs, os_window->num_tabs);
         os_window->tabs[os_window->num_tabs].id = ++global_state.tab_id_counter;
         os_window->tabs[os_window->num_tabs].border_rects.vao_idx = create_border_vao();
         return os_window->tabs[os_window->num_tabs++].id;
@@ -105,7 +104,7 @@ add_window(id_type os_window_id, id_type tab_id, PyObject *title) {
     WITH_TAB(os_window_id, tab_id);
         ensure_space_for(tab, windows, Window, tab->num_windows + 1, capacity, 1, true);
         make_os_window_context_current(osw);
-        memset(tab->windows + tab->num_windows, 0, sizeof(Window));
+        zero_at_i(tab->windows, tab->num_windows);
         tab->windows[tab->num_windows].id = ++global_state.window_id_counter;
         tab->windows[tab->num_windows].visible = true;
         tab->windows[tab->num_windows].title = title;
@@ -139,7 +138,7 @@ destroy_window(Window *w) {
 
 static inline void
 remove_window_inner(Tab *tab, id_type id) {
-    REMOVER(tab->windows, id, tab->num_windows, Window, destroy_window, tab->capacity);
+    REMOVER(tab->windows, id, tab->num_windows, destroy_window, tab->capacity);
 }
 
 static inline void
@@ -161,7 +160,7 @@ destroy_tab(Tab *tab) {
 static inline void
 remove_tab_inner(OSWindow *os_window, id_type id) {
     make_os_window_context_current(os_window);
-    REMOVER(os_window->tabs, id, os_window->num_tabs, Tab, destroy_tab, os_window->capacity);
+    REMOVER(os_window->tabs, id, os_window->num_tabs, destroy_tab, os_window->capacity);
 }
 
 static inline void
@@ -180,6 +179,7 @@ destroy_os_window_item(OSWindow *w) {
     Py_CLEAR(w->window_title); Py_CLEAR(w->tab_bar_render_data.screen);
     if (w->offscreen_texture_id) free_texture(&w->offscreen_texture_id);
     remove_vao(w->tab_bar_render_data.vao_idx);
+    remove_vao(w->gvao_idx);
     free(w->tabs); w->tabs = NULL;
 }
 
@@ -192,7 +192,7 @@ remove_os_window(id_type os_window_id) {
     END_WITH_OS_WINDOW
     if (found) {
         WITH_OS_WINDOW_REFS
-            REMOVER(global_state.os_windows, os_window_id, global_state.num_os_windows, OSWindow, destroy_os_window_item, global_state.capacity);
+            REMOVER(global_state.os_windows, os_window_id, global_state.num_os_windows, destroy_os_window_item, global_state.capacity);
         END_WITH_OS_WINDOW_REFS
         update_os_window_references();
     }
@@ -249,7 +249,7 @@ add_borders_rect(id_type os_window_id, id_type tab_id, uint32_t left, uint32_t t
 
 void
 os_window_regions(OSWindow *os_window, Region *central, Region *tab_bar) {
-    if (!global_state.tab_bar_hidden && os_window->num_tabs > 1) {
+    if (!global_state.tab_bar_hidden && os_window->num_tabs >= OPT(tab_bar_min_tabs)) {
         switch(OPT(tab_bar_edge)) {
             case TOP_EDGE:
                 central->left = 0; central->top = os_window->fonts_data->cell_height; central->right = os_window->viewport_width - 1;
@@ -265,7 +265,7 @@ os_window_regions(OSWindow *os_window, Region *central, Region *tab_bar) {
                 break;
         }
     } else {
-        memset(tab_bar, 0, sizeof(Region));
+        zero_at_ptr(tab_bar);
         central->left = 0; central->top = 0; central->right = os_window->viewport_width - 1;
         central->bottom = os_window->viewport_height - 1;
     }
@@ -289,8 +289,11 @@ os_window_regions(OSWindow *os_window, Region *central, Region *tab_bar) {
 #define KK5I(name) PYWRAP1(name) { id_type a, b; unsigned int c, d, e, f, g; PA("KKIIIII", &a, &b, &c, &d, &e, &f, &g); name(a, b, c, d, e, f, g); Py_RETURN_NONE; }
 #define BOOL_SET(name) PYWRAP1(set_##name) { global_state.name = PyObject_IsTrue(args); Py_RETURN_NONE; }
 
+static color_type default_color = 0;
+
 static inline color_type
 color_as_int(PyObject *color) {
+    if (color == Py_None && default_color) return default_color;
     if (!PyTuple_Check(color)) { PyErr_SetString(PyExc_TypeError, "Not a color tuple"); return 0; }
 #define I(n, s) ((PyLong_AsUnsignedLong(PyTuple_GET_ITEM(color, n)) & 0xff) << s)
     return (I(0, 16) | I(1, 8) | I(2, 0)) & 0xffffff;
@@ -302,10 +305,6 @@ repaint_delay(PyObject *val) {
     return (double)(PyLong_AsUnsignedLong(val)) / 1000.0;
 }
 
-#define dict_iter(d) { \
-    PyObject *key, *value; Py_ssize_t pos = 0; \
-    while (PyDict_Next(d, &pos, &key, &value))
-
 static int kitty_mod = 0;
 
 static inline int
@@ -321,6 +320,22 @@ convert_mods(PyObject *obj) {
     return resolve_mods(PyLong_AsLong(obj));
 }
 
+static MouseShape
+pointer_shape(PyObject *shape_name) {
+    const char *name = PyUnicode_AsUTF8(shape_name);
+    switch(name[0]) {
+        case 'a': return ARROW;
+        case 'h': return HAND;
+        case 'b': return BEAM;
+        default: break;
+    }
+    return BEAM;
+}
+
+#define dict_iter(d) { \
+    PyObject *key, *value; Py_ssize_t pos = 0; \
+    while (PyDict_Next(d, &pos, &key, &value))
+
 static inline void
 set_special_keys(PyObject *dict) {
     dict_iter(dict) {
@@ -351,6 +366,10 @@ PYWRAP1(set_options) {
     int is_wayland = 0, debug_gl = 0, debug_font_fallback = 0;
     PA("O|ppp", &opts, &is_wayland, &debug_gl, &debug_font_fallback);
     global_state.is_wayland = is_wayland ? true : false;
+#ifdef __APPLE__
+    global_state.has_render_frames = true;
+#endif
+    if (global_state.is_wayland) global_state.has_render_frames = true;
     global_state.debug_gl = debug_gl ? true : false;
     global_state.debug_font_fallback = debug_font_fallback ? true : false;
 #define GA(name) ret = PyObject_GetAttrString(opts, #name); if (ret == NULL) return NULL;
@@ -377,10 +396,15 @@ PYWRAP1(set_options) {
     S(touch_scroll_multiplier, PyFloat_AsDouble);
     S(open_url_modifiers, convert_mods);
     S(rectangle_select_modifiers, convert_mods);
+    S(terminal_select_modifiers, convert_mods);
     S(click_interval, PyFloat_AsDouble);
+    S(resize_debounce_time, PyFloat_AsDouble);
     S(url_color, color_as_int);
     S(background, color_as_int);
+    S(foreground, color_as_int);
+    default_color = 0x00ff00;
     S(active_border_color, color_as_int);
+    default_color = 0;
     S(inactive_border_color, color_as_int);
     S(bell_border_color, color_as_int);
     S(repaint_delay, repaint_delay);
@@ -388,12 +412,17 @@ PYWRAP1(set_options) {
     S(sync_to_monitor, PyObject_IsTrue);
     S(close_on_child_death, PyObject_IsTrue);
     S(window_alert_on_bell, PyObject_IsTrue);
-    S(macos_option_as_alt, PyObject_IsTrue);
+    S(macos_option_as_alt, PyLong_AsUnsignedLong);
     S(macos_traditional_fullscreen, PyObject_IsTrue);
     S(macos_quit_when_last_window_closed, PyObject_IsTrue);
+    S(macos_show_window_title_in_menubar, PyObject_IsTrue);
     S(macos_window_resizable, PyObject_IsTrue);
     S(macos_hide_from_tasks, PyObject_IsTrue);
     S(macos_thicken_font, PyFloat_AsDouble);
+    S(tab_bar_min_tabs, PyLong_AsUnsignedLong);
+    S(disable_ligatures, PyLong_AsLong);
+    S(resize_draw_strategy, PyLong_AsLong);
+    S(pointer_shape_when_grabbed, pointer_shape);
 
     GA(tab_bar_style);
     global_state.tab_bar_hidden = PyUnicode_CompareWithASCIIString(ret, "hidden") == 0 ? true: false;
@@ -682,8 +711,8 @@ PYWRAP1(patch_global_colors) {
 #define P(name) { \
     PyObject *val = PyDict_GetItemString(spec, #name); \
     if (val) { \
-		global_state.opts.name = PyLong_AsLong(val); \
-	} \
+        global_state.opts.name = PyLong_AsLong(val); \
+    } \
 }
     P(active_border_color); P(inactive_border_color); P(bell_border_color);
     if (configured) {
